@@ -10,6 +10,7 @@ pub use page_no::*;
 
 use file_ext::FileExt;
 use size_info::SizeInfo;
+use std::cell::UnsafeCell;
 use std::convert::TryInto;
 use std::fs::File;
 use std::io::{Error, ErrorKind, Result};
@@ -20,16 +21,19 @@ use tokio::task::spawn_blocking;
 
 /// **NOTE**: Caller must ensure that closure must not panic.
 /// If it does, it will be undefined behavior.
-macro_rules! into_async { [$body: expr] => { unsafe { spawn_blocking(move || $body).await.unwrap_unchecked() } }; }
+macro_rules! asyncify { [$body: expr] => { unsafe { spawn_blocking(move || $body).await.unwrap_unchecked() } }; }
 
 pub struct Pages<N, const SIZE: usize> {
     /// Total Page number
     len: AtomicU32,
     rwlock: RwLock<N>,
     file: &'static File,
-    metadata: [u8; SIZE],
+    metadata: UnsafeCell<[u8; SIZE]>,
 }
 
+unsafe impl<N: Send, const S: usize> Send for Pages<N, S> {}
+unsafe impl<N: Sync, const S: usize> Sync for Pages<N, S> {}
+
 impl<N: PageNo, const SIZE: usize> Pages<N, SIZE> {
     /// Create a new `Pages` instance.
     pub fn open(file: File) -> Result<Self> {
@@ -60,21 +64,21 @@ impl<N: PageNo, const SIZE: usize> Pages<N, SIZE> {
             file_len as u32 / SIZE as u32
         };
         Ok(Self {
-            len: AtomicU32::new(len),
-            file: Box::leak(Box::new(file)),
+            len: len.into(),
             rwlock: RwLock::new(),
-            metadata,
+            metadata: metadata.into(),
+            file: Box::leak(Box::new(file)),
         })
     }
 
     pub async fn sync_data(&self) -> Result<()> {
         let file = self.file;
-        into_async!(file.sync_data())
+        asyncify!(file.sync_data())
     }
 
     pub async fn sync_all(&self) -> Result<()> {
         let file = self.file;
-        into_async!(file.sync_all())
+        asyncify!(file.sync_all())
     }
 
     /// Read a page, without locking.
@@ -82,7 +86,7 @@ impl<N: PageNo, const SIZE: usize> Pages<N, SIZE> {
         debug_assert!(num.as_u32() < self.len());
         let num = num.as_u32() as u64;
         let file = self.file;
-        into_async!({
+        asyncify!({
             let mut buf = [0; SIZE];
             file.read_exact_at(&mut buf, SIZE as u64 * num)?;
             Ok(buf)
@@ -110,14 +114,14 @@ impl<N: PageNo, const SIZE: usize> Pages<N, SIZE> {
         debug_assert!(num.as_u32() < self.len());
         let num = num.as_u32() as u64;
         let file = self.file;
-        into_async!(file.write_all_at(&buf, SIZE as u64 * num))
+        asyncify!(file.write_all_at(&buf, SIZE as u64 * num))
     }
 
     /// Increase page number by `count`
     pub async fn alloc(&self, count: u32) -> Result<N> {
         let old_len = self.len.fetch_add(count, Ordering::SeqCst);
         let file = self.file;
-        into_async!(file.set_len(SIZE as u64 * (old_len + count) as u64))?;
+        asyncify!(file.set_len(SIZE as u64 * (old_len + count) as u64))?;
         Ok(N::new(old_len))
     }
 
@@ -144,14 +148,19 @@ impl<N: PageNo, const SIZE: usize> Pages<N, SIZE> {
         self.file
     }
 
+    #[allow(clippy::mut_from_ref)]
     /// Get a mutable reference to the pages's metadata.
-    pub fn metadata(&self) -> *mut [u8] {
-        &self.metadata[4..] as *const _ as *mut _
+    pub fn metadata(&self) -> &mut [u8] {
+        unsafe { &mut (*self.metadata.get())[4..] }
+    }
+
+    pub fn raw_lock_api(&self) -> &RwLock<N> {
+        &self.rwlock
     }
 }
 
 impl<N, const SIZE: usize> Drop for Pages<N, SIZE> {
     fn drop(&mut self) {
-        self.file.write_all_at(&self.metadata, 0).unwrap();
+        self.file.write_all_at(self.metadata.get_mut(), 0).unwrap();
     }
 }
