@@ -13,6 +13,10 @@ impl<'a, N: PageNo, const SIZE: usize> Page<'a, N, SIZE> {
     pub fn write(self) -> impl Future<Output = Result<()>> + 'a {
         self.pages.write(self.num, self.buf)
     }
+    
+    pub fn write_buf(self, buf: [u8; SIZE]) -> impl Future<Output = Result<()>> + 'a {
+        self.pages.write(self.num, buf)
+    }
 }
 
 impl<N: PageNo, const SIZE: usize> Deref for Page<'_, N, SIZE> {
